#include "../SDK/foobar2000.h"
#include "inttypes.h"
#include <a52.h>
#include "ac3_common.h"

int convert_output(audio_sample* src,
				   audio_sample * dst,
				   const int skip_samples,
				   const int flags)
{
	int i;
	audio_sample * dst_start = dst;

	if (flags & A52_LFE)
	{
		dst += 3;
		src += skip_samples;
		for (i = skip_samples; i < 256; i++)
		{
			*dst = *src++;
			dst += 6;
		}
		dst = dst_start;
		src += skip_samples;
	}
	else
	{
		src += skip_samples;
		switch (flags & A52_CHANNEL_MASK)
		{
		case A52_MONO:
			memcpy(dst, src, (256 - skip_samples) * sizeof(audio_sample));
			return 1;
		case A52_STEREO:
		case A52_CHANNEL:
		case A52_DOLBY:
			for (i = skip_samples; i < 256; i++)
			{
				*dst++ = *src;
				*dst++ = src[256];
				src++;
			}
			return 2;
		case A52_2F2R:
			for (i = skip_samples; i < 256; i++)
			{
				*dst++ = *src;
				*dst++ = src[256];
				*dst++ = src[512];
				*dst++ = src[768];
				src++;
			}
			return 4;
		}
	}
	switch (flags & A52_CHANNEL_MASK)
	{
	case A52_MONO:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = *src;
			*dst++ = *src++;
			dst += 3;
		}
		break;
	case A52_STEREO:
	case A52_CHANNEL:
	case A52_DOLBY:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[256];
			*dst++ = .5 * (*src + src[256]);
			dst += 3;
			src++;
		}
		break;
	case A52_3F:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[512];
			*dst++ = src[256];
			dst += 3;
			src++;
		}
		break;
	case A52_2F1R:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[256];
			dst += 2;
			*dst++ = src[512];
			*dst++ = src[512];
			src++;
		}
		break;
	case A52_3F1R:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[512];
			*dst++ = src[256];
			dst++;
			*dst++ = src[768];
			*dst++ = src[768];
			src++;
		}
		break;
	case A52_2F2R:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[256];
			*dst++ = .5 * (*src + src[256]);
			dst++;
			*dst++ = src[512];
			*dst++ = src[768];
			src++;
		}
		break;
	case A52_3F2R:
		for (i = skip_samples; i < 256; i++)
		{
			*dst++ = *src;
			*dst++ = src[512];
			*dst++ = src[256];
			dst++;
			*dst++ = src[768];
			*dst++ = src[1024];
			src++;
		}
		break;
	}
	return 6;
}