#include "../SDK/foobar2000.h"

#include "inttypes.h"
extern "C"
{
	#include <a52.h>
	#include <mm_accel.h>
}
#include "ac3_common.h"

#define PLUGIN_VERSION "0.9"
#define COPYRIGHT "Based on foo_ac3_pre0.7 by kode54\nPorted for 0.9 by Haru Ayana <ayana@reharmonize.net>\nBased on liba52 v0.7.4 (CVS snapshots)"

class packet_decoder_ac3 : public packet_decoder
{
public:
    packet_decoder_ac3()
    {
		m_State = NULL;
    }

    ~packet_decoder_ac3()
    {
		cleanup();
    }

	virtual t_size set_stream_property(const GUID & p_type,t_size p_param1,const void * p_param2,t_size p_param2size)
	{
		return 0;
	}

	virtual void get_info(file_info & p_info)
	{
		p_info.info_set_int("samplerate", m_SampleRate);
		p_info.info_set_int("channels", m_Channels);
		//p_info.info_set_int("bitspersample", 0);
		p_info.info_set("codec", "ATSC A/52");
		p_info.info_set("encoding", "lossy");
	}

	virtual unsigned get_max_frame_dependency() { return 0; }
	virtual double get_max_frame_dependency_time() { return 0; }

	virtual void reset_after_seek()
	{
	}

	virtual void decode(const void * p_buffer,t_size p_bytes,audio_chunk & p_chunk,abort_callback & p_abort)
	{
		unsigned char* inputbuffer = (unsigned char*)p_buffer;
		if(!m_OutBuffSize)
		{
			int frame_size = 0;
			unsigned i = 0;
			while(frame_size == 0 && i < p_bytes)
			{
				frame_size = a52_syncinfo (inputbuffer + i, &m_Flags, &m_SampleRate, &m_BitRate);
				i++;
			}
			if(!frame_size) return;
			//ConfigureChannelsMix(A52_3F2R | A52_LFE);
			ConfigureChannelsMix(m_Flags);
			//ConfigureChannelsMix(A52_DOLBY);
			m_OutBuffSize = 256 * 6 * m_Channels;
			m_SampleBuffer.set_size(m_OutBuffSize);
		}

		int samples = ProcessData(inputbuffer, inputbuffer + p_bytes, m_SampleBuffer.get_ptr());

		p_chunk.set_data(m_SampleBuffer.get_ptr(), samples / m_Channels, m_Channels, m_SampleRate);
	}

	virtual bool analyze_first_frame_supported() { return false; }

	virtual void analyze_first_frame(const void * p_buffer,t_size p_bytes,abort_callback & p_abort)
	{
		return;
	}

	void open(const GUID & p_owner,bool p_decode,t_size p_param1,const void * p_param2,t_size p_param2size,abort_callback & p_abort)
	{
		const matroska_setup * setup = (const matroska_setup*) p_param2;
		cleanup();
		m_Channels = setup->channels;
		m_SampleRate = setup->sample_rate;
		m_State = a52_init(MM_ACCEL_DJBFFT);
		if (m_State == NULL) throw exception_io_data();
	}

	static bool g_is_our_setup(const GUID & p_owner,t_size p_param1,const void * p_param2,t_size p_param2size)
	{
		if (p_owner == owner_matroska)
		{
			if (p_param2size == sizeof(matroska_setup))
			{
				const matroska_setup * setup = (const matroska_setup*) p_param2;
				if (!strncmp(setup->codec_id,"A_AC3",5)) {
					return true;
				}
				else return false;
			}
			else return false;
		}
		else return false;
	}

private:
	uint32_t ProcessData(uint8_t * start, uint8_t * end, audio_sample * outbuff)
	{
		static uint8_t buf[3840];
		static uint8_t * bufptr = buf;
		static uint8_t * bufpos = buf + 7;
		
		static int sample_rate;
		static int flags;
		int bit_rate;
		int len;
		uint32_t OutBuffIndex = 0;
		uint32_t SampleBlockSize = 256 * m_Channels;
		
		while (1)
		{
			len = end - start;
			if (!len)
				break;
			if (len > bufpos - bufptr)
				len = bufpos - bufptr;
			memcpy(bufptr, start, len);
			bufptr += len;
			start += len;
			if (bufptr == bufpos)
			{
				if (bufpos == buf + 7)
				{
					int length;
					
					length = a52_syncinfo(buf, &flags, &sample_rate, &bit_rate);
					if (!length)
					{
						for (bufptr = buf; bufptr < buf + 6; bufptr++)
							bufptr[0] = bufptr[1];
						continue;
					}
					bufpos = buf + length;
				}
				else
				{
					sample_t level = 1.0, bias = 0, *samples;
					int i;
					
					flags = m_Flags;
					
					if (a52_frame(m_State, buf, &flags, &level, bias))
						goto error;
					
					a52_dynrng(m_State, NULL, NULL);

					for (i = 0; i < 6; i++)
					{
						if (a52_block(m_State))
							goto error;
						
						samples = a52_samples(m_State);
						
						if(OutBuffIndex + SampleBlockSize > m_OutBuffSize)
						{
							// There is something wrong
							return OutBuffIndex;
						}
						
						convert_output(samples, outbuff + OutBuffIndex, 0, m_Flags);
						OutBuffIndex += SampleBlockSize;
					}
					bufptr = buf;
					bufpos = buf + 7;
					continue;
					
error:
					bufptr = buf;
					bufpos = buf + 7;
				}
			}
		}
		return OutBuffIndex;
	}

	void ConfigureChannelsMix(int flags)
	{
		static const uint8_t nfchans_tbl[] = {
			2, 1, 2, 3, 3, 4, 4, 5, 1, 1, 2
		};
		m_Flags = flags;
		m_Channels = nfchans_tbl[flags & A52_CHANNEL_MASK] + !!(flags & A52_LFE);
	}
	
	void cleanup() 
	{
		if (m_State)
		{
			a52_free(m_State);
			m_State = NULL;
		}
		m_OutBuffSize = 0;
	}

	a52_state_t* m_State;
	int32_t m_Flags;
	int32_t m_SampleRate;
	int32_t m_BitRate;
	uint32_t m_Channels;
	uint32_t m_OutBuffSize;

	pfc::array_t<audio_sample> m_SampleBuffer;
};

static packet_decoder_factory_t<packet_decoder_ac3> g_packet_decoder_ac3_factory;

DECLARE_COMPONENT_VERSION("ATSC A/52 (AC3) Packet Decoder", PLUGIN_VERSION, COPYRIGHT);